@@ -1,6 +1,5 @@
-
 # Copyright (c) 2022, CovidPyLib
-# This file is part of CovidPy v0.1.1.
+# This file is part of CovidPy v0.1.2
 #
 # The project has been distributed in the hope it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,6 +12,8 @@ import platform
 import zlib
 import qrcode
 import cbor2
+import importlib.resources as pkg_resources
+from . import certs
 
 try:
     import pyzbar.pyzbar
@@ -26,7 +27,7 @@ except ImportError as import_error:
         raise ImportError(
             """ERROR: pyzbar or zbar not found CovidPy won't work without it
             please install pyzbar using pip or zbar using your package manager.
-            example: 'sudo apt install libzbar0 on debian-based distros."""
+            example: sudo apt install libzbar0 on debian-based distros."""
         ) from import_error
     if platform.system() == "Darwin":
         raise ImportError(
@@ -56,22 +57,20 @@ from .verifier import DCCVerifier
 from .types import QRCode, VerifyResult, Certificate
 from .errors import InvalidDCC
 
-
 class CovidPy:
     def __init__(
         self,
         disable_keys_update: bool = False,
-        disable_blalcklist_update: bool = False,
+        disable_blacklist_update: bool = False,
         disable_blacklist: bool = False,
     ) -> None:
-        self.__autoblacklist = disable_blalcklist_update
+        self.__autoblacklist = disable_blacklist_update
         self.__autokids = disable_keys_update
         self.__disableblacklist = disable_blacklist
         self.__verifier = DCCVerifier(self.__autoblacklist, self.__autokids)
         self.__verifier.load_eu_keys()
         if not self.__disableblacklist:
             self.__verifier.load_blacklist()
-        self.certspath = "certs"
 
     def __decodecertificate(self, cert):
         img = PIL.Image.open(cert) if isinstance(cert, str) else cert.to_bytesio
@@ -86,8 +85,7 @@ class CovidPy:
         if cert.startswith("HC1:"):
             b45data = cert.replace("HC1:", "")
             compresseddata = b45decode(b45data)
-            decompressed = zlib.decompress(compresseddata)
-            return decompressed
+            return zlib.decompress(compresseddata)
 
         raise InvalidDCC(
             "The given code is not a DCC, check the 'details' attribute for more details",
@@ -97,6 +95,21 @@ class CovidPy:
     def __get_uvci(self, cert):
         if isinstance(cert, Certificate):
             return cert.certificate[0].certificate_identifier
+        elif isinstance(cert, dict):
+            for key, value in cert.items():
+                if isinstance(value, dict):
+                    if (val := self.__get_uvci(value)) is not None:
+                        return val
+                elif isinstance(value, list):
+                    for new_value in value:
+                        if isinstance(new_value, dict):
+                            ci_value = new_value.get("ci", None)
+                            return ci_value
+                elif isinstance(key, str):
+                    if key == "ci":
+                        return value
+                    return None
+            return None    
         else:
             raise InvalidDCC(
                 "The given code is not a DCC, check the 'details' attribute for more details",
@@ -115,16 +128,16 @@ class CovidPy:
 
     def __genqr(self, payload: dict):
         cbordata = cbor2.dumps(payload)
-        with open(f"{self.certspath}\\dsc-worker.pem", "rb") as file:
-            pem = file.read()
-            cert = x509.load_pem_x509_certificate(pem)
-            fingerprint = cert.fingerprint(hashes.SHA256())
-            keyid = fingerprint[0:8]
+        #with open(f"certs\\dsc-worker.pem", "rb") as file:
+        pem = pkg_resources.read_binary(certs, "dsc-worker.pem")
+        cert = x509.load_pem_x509_certificate(pem)
+        fingerprint = cert.fingerprint(hashes.SHA256())
+        keyid = fingerprint[:8]
 
-        with open(f"{self.certspath}\\dsc-worker.key", "rb") as file:
-            pem = file.read()
-            keyfile = load_pem_private_key(pem, password=None)
-            priv = keyfile.private_numbers().private_value.to_bytes(32, byteorder="big")
+        #with open(f"certs\\dsc-worker.key", "rb") as file:
+        pem = pkg_resources.read_binary(certs, "dsc-worker.key")
+        keyfile = load_pem_private_key(pem, password=None)
+        priv = keyfile.private_numbers().private_value.to_bytes(32, byteorder="big")
 
         msg = Sign1Message(phdr={Algorithm: Es256, KID: keyid}, payload=cbordata)
 
@@ -145,8 +158,7 @@ class CovidPy:
 
     def encode(self, data: dict) -> QRCode:
         gen_qr = self.__genqr(data)
-        qr_code = QRCode(gen_qr[0], gen_qr[1], self.__is_blacklisted(data), self)
-        return qr_code
+        return QRCode(gen_qr[0], gen_qr[1], self.__is_blacklisted(data), self)
 
     def verify(self, cert) -> VerifyResult:
         revoked = self.__is_blacklisted(self.decode(cert))
